---@type Tinkr
local Tinkr,
---@class Bastion
Bastion = ...

---@class Bastion.TimeToDieManager
---@field units table<string, Bastion.TimeToDieUnit>
---@field recycle Bastion.TimeToDieUnit[]
local TimeToDieManager = {}

local tokenPrefixes = {
    "^arena[1-5]",
    "^boss[1-5]",
    "^focus",
    "^mouseover",
    "^none",
    "^party[1-4]",
    "^partypet[1-4]",
    "^pet",
    "^player",
    "^raid[1-40]",
    "^raidpet[1-40]",
    "^target",
    "^vehicle",
    "^nameplate[1-40]",
    "^spectated",
}

---@param token string
local validateToken = function(token)
    for i = 1, #tokenPrefixes do
        if string.match(token, tokenPrefixes[i]) then
            return true
        end
    end
end

function TimeToDieManager:__index(k)
    if type(TimeToDieManager[k]) ~= "nil" then
        return TimeToDieManager[k]
    end

    local kguid = type(k) == "string" and validateToken(k) and ObjectGUID(k) or k

    if kguid and self.units[kguid] then
        return self.units[kguid]
    end

    if self.units[kguid] == nil then
        if Object(k) then
            --return self:SetObject(Bastion.Unit:New(Object(k) or k))
        end
    end
end

function TimeToDieManager:New()
    ---@class Bastion.TimeToDieManager
    local self = setmetatable({}, TimeToDieManager)
    self.units = {}
    self.recycle = {}
    return self
end

---@param unitId UnitIds
function TimeToDieManager:NAME_PLATE_UNIT_ADDED(unitId)
    print("NAME_PLATE_UNIT_ADDED", unitId)
end

---@param unitId UnitIds
function TimeToDieManager:NAME_PLATE_UNIT_REMOVED(unitId)
    print("NAME_PLATE_UNIT_REMOVED", unitId)
end

---@param unitId UnitIds
function TimeToDieManager:UNIT_FLAGS(unitId)
    print("UNIT_FLAGS", unitId)
end

function TimeToDieManager:RegisterEvents()
    Bastion.EventManager:RegisterWoWEvent("NAME_PLATE_UNIT_ADDED", function(...)
        self:NAME_PLATE_UNIT_ADDED(...)
    end)

    Bastion.EventManager:RegisterWoWEvent("NAME_PLATE_UNIT_REMOVED", function(...)
        self:NAME_PLATE_UNIT_REMOVED(...)
    end)

    Bastion.EventManager:RegisterWoWEvent("UNIT_FLAGS", function(...)
        self:UNIT_FLAGS(...)
    end)
end

---@param unitGuid string
function TimeToDieManager:GetUnit(unitGuid)
    if unitGuid and self.units[unitGuid] then
        return self.units[unitGuid]
    end
end

---@param unitGuid string
function TimeToDieManager:RemoveUnit(unitGuid)
    if type(self.units[unitGuid]) ~= "nil" then
        local oldUnit = self.units[unitGuid]
        self.units[unitGuid] = nil
        table.insert(self.recycle, oldUnit:Recycle())
    end
end

---@param when number
---@param healthPct number
---@param unit? WowGameObject
function TimeToDieManager:RecycleUnit(when, healthPct, unit)
    return Bastion.TimeToDieUnit:New(table.remove(self.recycle) or false, when, healthPct, unit)
end

---@param unitGuid string
---@param healthPct number
---@param when? number
function TimeToDieManager:UpdateUnit(unitGuid, healthPct, when)
    local unitObject = self.units[unitGuid]
    when = when or GetTime()
    if not unitObject then
        unitObject = self:GetUnit(unitGuid)
    end
end

Bastion.TimeToDieManager = TimeToDieManager:New()
